@@ -47,7 +47,6 @@
  */
 
 #include "pg.h"
-#include "pg_config.h"
 
 int pg_skip_deprecation_warning;
 VALUE rb_mPG;
@@ -127,26 +126,6 @@ const char * const (pg_enc_pg2ruby_mapping[][2]) = {
 static struct st_table *enc_pg2ruby;
 
 
-/*
- * Look up the JOHAB encoding, creating it as a dummy encoding if it's not
- * already defined.
- */
-static rb_encoding *
-pg_find_or_create_johab(void)
-{
-	static const char * const aliases[] = { "JOHAB", "Windows-1361", "CP1361" };
-	int enc_index;
-	size_t i;
-
-	for (i = 0; i < sizeof(aliases)/sizeof(aliases[0]); ++i) {
-		enc_index = rb_enc_find_index(aliases[i]);
-		if (enc_index > 0) return rb_enc_from_index(enc_index);
-	}
-
-	enc_index = rb_define_dummy_encoding(aliases[0]);
-	return rb_enc_from_index(enc_index);
-}
-
 /*
  * Return the given PostgreSQL encoding ID as an rb_encoding.
  *
@@ -187,10 +166,6 @@ pg_get_pg_encname_as_rb_encoding( const char *pg_encname )
 			return rb_enc_find( pg_enc_pg2ruby_mapping[i][1] );
 	}
 
-	/* JOHAB isn't a builtin encoding, so make up a dummy encoding if it's seen */
-	if ( strncmp(pg_encname, "JOHAB", 5) == 0 )
-		return pg_find_or_create_johab();
-
 	/* Fallthrough to ASCII-8BIT */
 	return rb_ascii8bit_encoding();
 }
@@ -377,7 +352,7 @@ pg_s_init_ssl(VALUE self, VALUE do_ssl)
  **************************************************************************/
 
 void
-Init_pg_ext()
+Init_pg_ext(void)
 {
 	if( RTEST(rb_eval_string("ENV['PG_SKIP_DEPRECATION_WARNING']")) ){
 		/* Set all bits to disable all deprecation warnings. */
@@ -704,6 +679,9 @@ Init_pg_ext()
 	rb_define_const(rb_mPGconstants, "INVALID_OID", INT2FIX(InvalidOid));
 	rb_define_const(rb_mPGconstants, "InvalidOid", INT2FIX(InvalidOid));
 
+	/* PostgreSQL compiled in default port */
+	rb_define_const(rb_mPGconstants, "DEF_PGPORT", INT2FIX(DEF_PGPORT));
+
 	/* Add the constants to the toplevel namespace */
 	rb_include_module( rb_mPG, rb_mPGconstants );
 
