@@ -107,17 +107,34 @@ pgresult_approx_size(const PGresult *result)
  * GC Mark function
  */
 static void
-pgresult_gc_mark( t_pg_result *this )
+pgresult_gc_mark( void *_this )
 {
+	t_pg_result *this = (t_pg_result *)_this;
 	int i;
 
-	rb_gc_mark( this->connection );
-	rb_gc_mark( this->typemap );
-	rb_gc_mark( this->tuple_hash );
-	rb_gc_mark( this->field_map );
+	rb_gc_mark_movable( this->connection );
+	rb_gc_mark_movable( this->typemap );
+	rb_gc_mark_movable( this->tuple_hash );
+	rb_gc_mark_movable( this->field_map );
 
 	for( i=0; i < this->nfields; i++ ){
-		rb_gc_mark( this->fnames[i] );
+		rb_gc_mark_movable( this->fnames[i] );
+	}
+}
+
+static void
+pgresult_gc_compact( void *_this )
+{
+	t_pg_result *this = (t_pg_result *)_this;
+	int i;
+
+	pg_gc_location( this->connection );
+	pg_gc_location( this->typemap );
+	pg_gc_location( this->tuple_hash );
+	pg_gc_location( this->field_map );
+
+	for( i=0; i < this->nfields; i++ ){
+		pg_gc_location( this->fnames[i] );
 	}
 }
 
@@ -125,8 +142,9 @@ pgresult_gc_mark( t_pg_result *this )
  * GC Free function
  */
 static void
-pgresult_clear( t_pg_result *this )
+pgresult_clear( void *_this )
 {
+	t_pg_result *this = (t_pg_result *)_this;
 	if( this->pgresult && !this->autoclear ){
 		PQclear(this->pgresult);
 #ifdef HAVE_RB_GC_ADJUST_MEMORY_USAGE
@@ -139,15 +157,17 @@ pgresult_clear( t_pg_result *this )
 }
 
 static void
-pgresult_gc_free( t_pg_result *this )
+pgresult_gc_free( void *_this )
 {
+	t_pg_result *this = (t_pg_result *)_this;
 	pgresult_clear( this );
 	xfree(this);
 }
 
 static size_t
-pgresult_memsize( t_pg_result *this )
+pgresult_memsize( const void *_this )
 {
+	const t_pg_result *this = (const t_pg_result *)_this;
 	/* Ideally the memory 'this' is pointing to should be taken into account as well.
 	 * However we don't want to store two memory sizes in t_pg_result just for reporting by ObjectSpace.memsize_of.
 	 */
@@ -155,16 +175,15 @@ pgresult_memsize( t_pg_result *this )
 }
 
 static const rb_data_type_t pgresult_type = {
-	"pg",
+	"PG::Result",
 	{
-		(void (*)(void*))pgresult_gc_mark,
-		(void (*)(void*))pgresult_gc_free,
-		(size_t (*)(const void *))pgresult_memsize,
+		pgresult_gc_mark,
+		pgresult_gc_free,
+		pgresult_memsize,
+		pg_compact_callback(pgresult_gc_compact),
 	},
 	0, 0,
-#ifdef RUBY_TYPED_FREE_IMMEDIATELY
 	RUBY_TYPED_FREE_IMMEDIATELY,
-#endif
 };
 
 /* Needed by sequel_pg gem, do not delete */
@@ -191,7 +210,7 @@ pg_new_result2(PGresult *result, VALUE rb_pgconn)
 	this->pgresult = result;
 	this->connection = rb_pgconn;
 	this->typemap = pg_typemap_all_strings;
-	this->p_typemap = DATA_PTR( this->typemap );
+	this->p_typemap = RTYPEDDATA_DATA( this->typemap );
 	this->nfields = -1;
 	this->tuple_hash = Qnil;
 	this->field_map = Qnil;
@@ -202,11 +221,11 @@ pg_new_result2(PGresult *result, VALUE rb_pgconn)
 		t_pg_connection *p_conn = pg_get_connection(rb_pgconn);
 		VALUE typemap = p_conn->type_map_for_results;
 		/* Type check is done when assigned to PG::Connection. */
-		t_typemap *p_typemap = DATA_PTR(typemap);
+		t_typemap *p_typemap = RTYPEDDATA_DATA(typemap);
 
 		this->enc_idx = p_conn->enc_idx;
 		this->typemap = p_typemap->funcs.fit_to_result( typemap, self );
-		this->p_typemap = DATA_PTR( this->typemap );
+		this->p_typemap = RTYPEDDATA_DATA( this->typemap );
 		this->flags = p_conn->flags;
 	} else {
 		this->enc_idx = rb_locale_encindex();
@@ -270,7 +289,11 @@ pg_new_result_autoclear(PGresult *result, VALUE rb_pgconn)
  * call-seq:
  *    res.check -> nil
  *
- * Raises appropriate exception if PG::Result is in a bad state.
+ * Raises appropriate exception if PG::Result is in a bad state, which is:
+ * * +PGRES_BAD_RESPONSE+
+ * * +PGRES_FATAL_ERROR+
+ * * +PGRES_NONFATAL_ERROR+
+ * * +PGRES_PIPELINE_ABORTED+
  */
 VALUE
 pg_result_check( VALUE self )
@@ -295,10 +318,16 @@ pg_result_check( VALUE self )
 		case PGRES_SINGLE_TUPLE:
 		case PGRES_EMPTY_QUERY:
 		case PGRES_COMMAND_OK:
+#ifdef HAVE_PQENTERPIPELINEMODE
+		case PGRES_PIPELINE_SYNC:
+#endif
 			return self;
 		case PGRES_BAD_RESPONSE:
 		case PGRES_FATAL_ERROR:
 		case PGRES_NONFATAL_ERROR:
+#ifdef HAVE_PQENTERPIPELINEMODE
+		case PGRES_PIPELINE_ABORTED:
+#endif
 			error = rb_str_new2( PQresultErrorMessage(this->pgresult) );
 			break;
 		default:
@@ -353,7 +382,7 @@ pg_result_clear(VALUE self)
  * call-seq:
  *    res.cleared?      -> boolean
  *
- * Returns +true+ if the backend result memory has been free'd.
+ * Returns +true+ if the backend result memory has been freed.
  */
 VALUE
 pgresult_cleared_p( VALUE self )
@@ -367,7 +396,7 @@ pgresult_cleared_p( VALUE self )
  *    res.autoclear?      -> boolean
  *
  * Returns +true+ if the underlying C struct will be cleared at the end of a callback.
- * This applies only to Result objects received by the block to PG::Cinnection#set_notice_receiver .
+ * This applies only to Result objects received by the block to PG::Connection#set_notice_receiver .
  *
  * All other Result objects are automatically cleared by the GC when the object is no longer in use or manually by PG::Result#clear .
  *
@@ -495,6 +524,9 @@ static void pgresult_init_fnames(VALUE self)
  * * +PGRES_NONFATAL_ERROR+
  * * +PGRES_FATAL_ERROR+
  * * +PGRES_COPY_BOTH+
+ * * +PGRES_SINGLE_TUPLE+
+ * * +PGRES_PIPELINE_SYNC+
+ * * +PGRES_PIPELINE_ABORTED+
  */
 static VALUE
 pgresult_result_status(VALUE self)
@@ -506,7 +538,7 @@ pgresult_result_status(VALUE self)
  * call-seq:
  *    res.res_status( status ) -> String
  *
- * Returns the string representation of status +status+.
+ * Returns the string representation of +status+.
  *
 */
 static VALUE
@@ -1003,7 +1035,7 @@ pgresult_cmd_tuples(VALUE self)
 {
 	long n;
 	n = strtol(PQcmdTuples(pgresult_get(self)),NULL, 10);
-	return INT2NUM(n);
+	return LONG2NUM(n);
 }
 
 /*
@@ -1325,14 +1357,11 @@ pgresult_type_map_set(VALUE self, VALUE typemap)
 	t_pg_result *this = pgresult_get_this(self);
 	t_typemap *p_typemap;
 
-	if ( !rb_obj_is_kind_of(typemap, rb_cTypeMap) ) {
-		rb_raise( rb_eTypeError, "wrong argument type %s (expected kind of PG::TypeMap)",
-				rb_obj_classname( typemap ) );
-	}
-	Data_Get_Struct(typemap, t_typemap, p_typemap);
+	/* Check type of method param */
+	TypedData_Get_Struct(typemap, t_typemap, &pg_typemap_type, p_typemap);
 
 	this->typemap = p_typemap->funcs.fit_to_result( typemap, self );
-	this->p_typemap = DATA_PTR( this->typemap );
+	this->p_typemap = RTYPEDDATA_DATA( this->typemap );
 
 	return typemap;
 }
@@ -1353,22 +1382,21 @@ pgresult_type_map_get(VALUE self)
 }
 
 
-static void
-yield_hash(VALUE self, int ntuples, int nfields)
+static int
+yield_hash(VALUE self, int ntuples, int nfields, void *data)
 {
 	int tuple_num;
-	t_pg_result *this = pgresult_get_this(self);
 	UNUSED(nfields);
 
 	for(tuple_num = 0; tuple_num < ntuples; tuple_num++) {
 		rb_yield(pgresult_aref(self, INT2NUM(tuple_num)));
 	}
 
-	pgresult_clear( this );
+	return 1; /* clear the result */
 }
 
-static void
-yield_array(VALUE self, int ntuples, int nfields)
+static int
+yield_array(VALUE self, int ntuples, int nfields, void *data)
 {
 	int row;
 	t_pg_result *this = pgresult_get_this(self);
@@ -1384,11 +1412,11 @@ yield_array(VALUE self, int ntuples, int nfields)
 		rb_yield( rb_ary_new4( nfields, row_values ));
 	}
 
-	pgresult_clear( this );
+	return 1; /* clear the result */
 }
 
-static void
-yield_tuple(VALUE self, int ntuples, int nfields)
+static int
+yield_tuple(VALUE self, int ntuples, int nfields, void *data)
 {
 	int tuple_num;
 	t_pg_result *this = pgresult_get_this(self);
@@ -1405,10 +1433,12 @@ yield_tuple(VALUE self, int ntuples, int nfields)
 		VALUE tuple = pgresult_tuple(copy, INT2FIX(tuple_num));
 		rb_yield( tuple );
 	}
+	return 0; /* don't clear the result */
 }
 
-static VALUE
-pgresult_stream_any(VALUE self, void (*yielder)(VALUE, int, int))
+/* Non-static, and data pointer for use by sequel_pg */
+VALUE
+pgresult_stream_any(VALUE self, int (*yielder)(VALUE, int, int, void*), void* data)
 {
 	t_pg_result *this;
 	int nfields;
@@ -1427,6 +1457,7 @@ pgresult_stream_any(VALUE self, void (*yielder)(VALUE, int, int))
 
 		switch( PQresultStatus(pgresult) ){
 			case PGRES_TUPLES_OK:
+			case PGRES_COMMAND_OK:
 				if( ntuples == 0 )
 					return self;
 				rb_raise( rb_eInvalidResultStatus, "PG::Result is not in single row mode");
@@ -1436,14 +1467,21 @@ pgresult_stream_any(VALUE self, void (*yielder)(VALUE, int, int))
 				pg_result_check( self );
 		}
 
-		yielder( self, ntuples, nfields );
+		if( yielder( self, ntuples, nfields, data ) ){
+			pgresult_clear( this );
+		}
+
+		if( gvl_PQisBusy(pgconn) ){
+			/* wait for input (without blocking) before reading each result */
+			pgconn_block( 0, NULL, this->connection );
+		}
 
 		pgresult = gvl_PQgetResult(pgconn);
 		if( pgresult == NULL )
-			rb_raise( rb_eNoResultError, "no result received - possibly an intersection with another result retrieval");
+			rb_raise( rb_eNoResultError, "no result received - possibly an intersection with another query");
 
 		if( nfields != PQnfields(pgresult) )
-			rb_raise( rb_eInvalidChangeOfResultFields, "number of fields must not change in single row mode");
+			rb_raise( rb_eInvalidChangeOfResultFields, "number of fields changed in single row mode from %d to %d - this is a sign for intersection with another query", nfields, PQnfields(pgresult));
 
 		this->pgresult = pgresult;
 	}
@@ -1487,7 +1525,7 @@ pgresult_stream_any(VALUE self, void (*yielder)(VALUE, int, int))
 static VALUE
 pgresult_stream_each(VALUE self)
 {
-	return pgresult_stream_any(self, yield_hash);
+	return pgresult_stream_any(self, yield_hash, NULL);
 }
 
 /*
@@ -1503,7 +1541,7 @@ pgresult_stream_each(VALUE self)
 static VALUE
 pgresult_stream_each_row(VALUE self)
 {
-	return pgresult_stream_any(self, yield_array);
+	return pgresult_stream_any(self, yield_array, NULL);
 }
 
 /*
@@ -1520,7 +1558,7 @@ pgresult_stream_each_tuple(VALUE self)
 	/* allocate VALUEs that are shared between all streamed tuples */
 	ensure_init_for_tuple(self);
 
-	return pgresult_stream_any(self, yield_tuple);
+	return pgresult_stream_any(self, yield_tuple, NULL);
 }
 
 /*
@@ -1531,7 +1569,7 @@ pgresult_stream_each_tuple(VALUE self)
  * It can be set to one of:
  * * +:string+ to use String based field names
  * * +:symbol+ to use Symbol based field names
- * * +:static_symbol+ to use pinned Symbol (can not be garbage collected) - Don't use this, it will probably removed in future.
+ * * +:static_symbol+ to use pinned Symbol (can not be garbage collected) - Don't use this, it will probably be removed in future.
  *
  * The default is retrieved from PG::Connection#field_name_type , which defaults to +:string+ .
  *
@@ -1581,13 +1619,14 @@ pgresult_field_name_type_get(VALUE self)
 }
 
 void
-init_pg_result()
+init_pg_result(void)
 {
 	sym_string = ID2SYM(rb_intern("string"));
 	sym_symbol = ID2SYM(rb_intern("symbol"));
 	sym_static_symbol = ID2SYM(rb_intern("static_symbol"));
 
-	rb_cPGresult = rb_define_class_under( rb_mPG, "Result", rb_cData );
+	rb_cPGresult = rb_define_class_under( rb_mPG, "Result", rb_cObject );
+	rb_undef_alloc_func(rb_cPGresult);
 	rb_include_module(rb_cPGresult, rb_mEnumerable);
 	rb_include_module(rb_cPGresult, rb_mPGconstants);
 
